import throttle from 'lodash.throttle';
import raf from 'raf';
import { intBetween, getCoords } from './util.js';
var ScrollingMonitor = /*#__PURE__*/function () {
  function ScrollingMonitor(dragDropManager, container, options) {
    var _this = this;
    this.handleEvent = function (evt) {
      if (_this.dragging && !_this.attached) {
        _this.attach();
        _this.updateScrolling(evt);
      }
    };
    // Update scaleX and scaleY every 100ms or so
    // and start scrolling if necessary
    this.updateScrolling = throttle(function (evt) {
      var _this$container$getBo = _this.container.getBoundingClientRect(),
        x = _this$container$getBo.left,
        y = _this$container$getBo.top,
        w = _this$container$getBo.width,
        h = _this$container$getBo.height;
      var box = {
        x: x,
        y: y,
        w: w,
        h: h
      };
      var coords = getCoords(evt);

      // calculate strength
      _this.scaleX = _this.options.horizontalStrength(box, coords);
      _this.scaleY = _this.options.verticalStrength(box, coords);

      // start scrolling if we need to
      if (!_this.frame && (_this.scaleX || _this.scaleY)) {
        _this.startScrolling();
      }
    }, 100, {
      trailing: false
    });
    this.dragDropManager = dragDropManager;
    this.container = container;
    this.options = options;
    this.scaleX = 0;
    this.scaleY = 0;
    this.frame = null;
    this.attached = false;
    this.dragging = false;
  }
  var _proto = ScrollingMonitor.prototype;
  _proto.start = function start() {
    var _this2 = this;
    this.container.addEventListener('dragover', this.handleEvent);
    // touchmove events don't seem to work across siblings, so we unfortunately
    // have to attach the listeners to the body
    window.document.body.addEventListener('touchmove', this.handleEvent);
    this.clearMonitorSubscription = this.dragDropManager.getMonitor().subscribeToStateChange(function () {
      return _this2.handleMonitorChange();
    });
  };
  _proto.stop = function stop() {
    this.container.removeEventListener('dragover', this.handleEvent);
    window.document.body.removeEventListener('touchmove', this.handleEvent);
    this.clearMonitorSubscription();
    this.stopScrolling();
  };
  _proto.handleMonitorChange = function handleMonitorChange() {
    var isDragging = this.dragDropManager.getMonitor().isDragging();
    if (!this.dragging && isDragging) {
      this.dragging = true;
    } else if (this.dragging && !isDragging) {
      this.dragging = false;
      this.stopScrolling();
    }
  };
  _proto.attach = function attach() {
    this.attached = true;
    window.document.body.addEventListener('dragover', this.updateScrolling);
    window.document.body.addEventListener('touchmove', this.updateScrolling);
  };
  _proto.detach = function detach() {
    this.attached = false;
    window.document.body.removeEventListener('dragover', this.updateScrolling);
    window.document.body.removeEventListener('touchmove', this.updateScrolling);
  };
  _proto.startScrolling = function startScrolling() {
    var _this3 = this;
    var i = 0;
    var tick = function tick() {
      var scaleX = _this3.scaleX,
        scaleY = _this3.scaleY,
        container = _this3.container;
      var _this3$options = _this3.options,
        strengthMultiplier = _this3$options.strengthMultiplier,
        onScrollChange = _this3$options.onScrollChange;

      // stop scrolling if there's nothing to do
      if (strengthMultiplier === 0 || scaleX + scaleY === 0) {
        _this3.stopScrolling();
        return;
      }

      // there's a bug in safari where it seems like we can't get
      // mousemove events from a container that also emits a scroll
      // event that same frame. So we double the strengthMultiplier and only adjust
      // the scroll position at 30fps
      if (i++ % 2) {
        var scrollLeft = container.scrollLeft,
          scrollTop = container.scrollTop,
          scrollWidth = container.scrollWidth,
          scrollHeight = container.scrollHeight,
          clientWidth = container.clientWidth,
          clientHeight = container.clientHeight;
        var newLeft = scaleX ? container.scrollLeft = intBetween(0, scrollWidth - clientWidth, scrollLeft + scaleX * strengthMultiplier) : scrollLeft;
        var newTop = scaleY ? container.scrollTop = intBetween(0, scrollHeight - clientHeight, scrollTop + scaleY * strengthMultiplier) : scrollTop;
        onScrollChange(newLeft, newTop);
      }
      _this3.frame = raf(tick);
    };
    tick();
  };
  _proto.stopScrolling = function stopScrolling() {
    this.detach();
    this.scaleX = 0;
    this.scaleY = 0;
    if (this.frame) {
      raf.cancel(this.frame);
      this.frame = null;
    }
  };
  return ScrollingMonitor;
}();
export { ScrollingMonitor as default };